getPdfpars.azzalini <-
function(p,model=0){
    CC <- function(x,alpha){return(pnorm(x)-2*OwenT(x,alpha))}
    target=c(pnorm(-1),0.5,pnorm(1.0))

  if(!is.na(p['V'])){
    V=as.numeric(p['V'])
    mu=as.numeric(p['mu'])
    gamma=as.numeric(p['gamma'])
    thing=2*gamma/(4-pi)
    omegadelta=sqrt(pi/2)*sign(thing)*abs(thing)**(1/3.0)
    omega=sqrt(V+2*omegadelta**2/pi)
    delta=omegadelta/omega
    if(abs(delta)>1) { warning("Skew too large for azzalini: null returned")
                            return(NULL)
                            }
    xi=mu-omegadelta*sqrt(2/pi)
    alpha=delta/sqrt(1-delta**2)
    guess=c(0,0,0)
    for(j  in 1:3){
       for(i in 1:NTRY){
         old=guess[i]
         val=pnorm(guess[j])-2*OwenT(guess[j],alpha)
         if(abs(target[j]-val) < SMALL) break
         diff=2*dnorm(guess[j])*pnorm(alpha*guess[j])
         guess[j]=guess[j]+(target[j]-val)/diff
    }
    if(i >= NTRY) warning("no convergence")
    }
    M=xi+omega*guess[2]
    sm=omega*(guess[2]-guess[1])
    sp=omega*(guess[3]-guess[2])
    
    ans=c(p,xi=xi,alpha=alpha,omega=omega,M=M,sp=sp,sm=sm)
    } else{ # get parameters from quantiles
    sp=as.numeric(p['sp'])
    sm=as.numeric(p['sm'])
    M=as.numeric(p['M'])
    A=(sp-sm)/(sp+sm)

    getasym <- function(alpha,A){ # adjust alpha to get desired asymmetry
      for(i in 1:NTRY){
         got=as.numeric(lapply(guess,CC,alpha))
         # got=pnorm(guess)-2*OwenT(guess,alpha)
         check=max(abs(target-got))
         if(check<SMALL) break
         diff=skewNormal(guess,alpha)
         sss=(target-got)/diff
         big=abs(sss)>0.2
         sss[big]=0.2*sign(sss[big])
         guess <<- guess+sss
         }
       if(i>=NTRY) {warning("Cannot determine asymmetry in Azzalini method"); return(NULL)}
       return((guess[1]+guess[3]-2*guess[2])/(guess[3]-guess[1]) -A)
       }
     guess <- c(-1,0,1)
     if(getasym(0,abs(A))*getasym(100,abs(A)) >0) {warning("Asymmetry too large for Azzalini: null returned");return(NULL)} # azzalini can't handle this
     u=uniroot(getasym,c(0,100),tol=1.0E-8,abs(A))
     alpha=as.numeric(u$root * sign(A))
     delta=alpha/sqrt(1+alpha**2)
     omega=as.numeric((sp+sm)/(guess[3]-guess[1]))
     xi=as.numeric(M-sign(A)*guess[2]*omega)
     ans=c(p,xi=xi,omega=omega,alpha=alpha,mu=xi+omega*delta*sqrt(2/pi),
       V=omega**2*(1-2*delta^2/pi),gamma=0.5*(4-pi)*(omega*delta*sqrt(2/pi))**3)
      }      
  class(ans) <- "azzalini"
  return(ans)
}
